@@ -3,7 +3,7 @@ import logging
 import os
 import time
 
-from oauthlib.oauth2 import TokenExpiredError, WebApplicationClient, BackendApplicationClient
+from oauthlib.oauth2 import TokenExpiredError, WebApplicationClient, BackendApplicationClient, LegacyApplicationClient
 from requests import Session
 from requests.adapters import HTTPAdapter
 from requests.exceptions import HTTPError, RequestException, ProxyError
@@ -35,6 +35,7 @@ DEFAULT_SCOPES = {
     'basic': [('offline_access',), 'User.Read'],
     'mailbox': ['Mail.Read'],
     'mailbox_shared': ['Mail.Read.Shared'],
+    "mailbox_settings": ["MailboxSettings.ReadWrite"],
     'message_send': ['Mail.Send'],
     'message_send_shared': ['Mail.Send.Shared'],
     'message_all': ['Mail.ReadWrite', 'Mail.Send'],
@@ -322,10 +323,11 @@ class Connection:
 
     def __init__(self, credentials, *, scopes=None,
                  proxy_server=None, proxy_port=8080, proxy_username=None,
-                 proxy_password=None, requests_delay=200, raise_http_errors=True,
+                 proxy_password=None, proxy_http_only=False, requests_delay=200, raise_http_errors=True,
                  request_retries=3, token_backend=None,
                  tenant_id='common',
                  auth_flow_type='authorization',
+                 username=None, password=None,
                  timeout=None, json_encoder=None,
                  verify_ssl=True, **kwargs):
         """ Creates an API connection object
@@ -355,30 +357,36 @@ class Connection:
             - 'authorization': 2 step web style grant flow using an authentication url
             - 'public': 2 step web style grant flow using an authentication url for public apps where
             client secret cannot be secured
-            - 'credentials': also called client credentials grant flow using only the cliend id and secret
+            - 'credentials': also called client credentials grant flow using only the client id and secret
+            - 'certificate': like credentials, but using the client id and a JWT assertion (obtained from a certificate)
+        :param str username: The user's email address to provide in case of auth_flow_type == 'password'
+        :param str password: The user's password to provide in case of auth_flow_type == 'password'
         :param float or tuple timeout: How long to wait for the server to send
             data before giving up, as a float, or a tuple (connect timeout, read timeout)
-        :param JSONEncoder json_encoder: The JSONEnocder to use during the JSON serialization on the request.
+        :param JSONEncoder json_encoder: The JSONEncoder to use during the JSON serialization on the request.
         :param bool verify_ssl: set the verify flag on the requests library
         :param dict kwargs: any extra params passed to Connection
         :raises ValueError: if credentials is not tuple of
          (client_id, client_secret)
         """
-        if auth_flow_type == 'public':  # allow client id only for public flow
+        if auth_flow_type in ('public', 'password'):  # allow client id only for public or password flow
             if isinstance(credentials, str): 
                 credentials = (credentials,)
             if not isinstance(credentials, tuple) or len(credentials) != 1 or (not credentials[0]):
-                raise ValueError('Provide client id only for public flow credentials')
+                raise ValueError('Provide client id only for public or password flow credentials')
         else:
             if not isinstance(credentials, tuple) or len(credentials) != 2 or (not credentials[0] and not credentials[1]):
                 raise ValueError('Provide valid auth credentials')
 
-        self._auth_flow_type = auth_flow_type  # 'authorization' or 'credentials' or 'public'
-        if auth_flow_type == 'credentials' and tenant_id == 'common':
-            raise ValueError('When using the "credentials" auth_flow the "tenant_id" must be set')
+        self._auth_flow_type = auth_flow_type  # 'authorization', 'credentials', 'certificate', 'password', or 'public'
+        if auth_flow_type in ('credentials', 'certificate', 'password') and tenant_id == 'common':
+            raise ValueError('When using the "credentials", "certificate", or "password" auth_flow the "tenant_id" '
+                             'must be set')
 
         self.tenant_id = tenant_id
         self.auth = credentials
+        self.username = username
+        self.password = password
         self.scopes = scopes
         self.store_token = True
         token_backend = token_backend or FileSystemTokenBackend(**kwargs)
@@ -388,7 +396,7 @@ class Connection:
         self.session = None  # requests Oauth2Session object
 
         self.proxy = {}
-        self.set_proxy(proxy_server, proxy_port, proxy_username, proxy_password)
+        self.set_proxy(proxy_server, proxy_port, proxy_username, proxy_password, proxy_http_only)
         self.requests_delay = requests_delay or 0
         self._previous_request_at = None  # store previous request time
         self.raise_http_errors = raise_http_errors
@@ -410,7 +418,7 @@ class Connection:
         return self._auth_flow_type
 
     def set_proxy(self, proxy_server, proxy_port, proxy_username,
-                  proxy_password):
+                  proxy_password, proxy_http_only):
         """ Sets a proxy on the Session
 
         :param str proxy_server: the proxy server
@@ -420,20 +428,23 @@ class Connection:
         """
         if proxy_server and proxy_port:
             if proxy_username and proxy_password:
+                proxy_uri = "{}:{}@{}:{}".format(proxy_username,
+                                                proxy_password,
+                                                proxy_server,
+                                                proxy_port)
+            else:
+                proxy_uri = "{}:{}".format(proxy_server,
+                                            proxy_port)
+            
+            if proxy_http_only is False:
                 self.proxy = {
-                    "http": "http://{}:{}@{}:{}".format(proxy_username,
-                                                        proxy_password,
-                                                        proxy_server,
-                                                        proxy_port),
-                    "https": "https://{}:{}@{}:{}".format(proxy_username,
-                                                          proxy_password,
-                                                          proxy_server,
-                                                          proxy_port),
+                    "http": "http://{}".format(proxy_uri),
+                    "https": "https://{}".format(proxy_uri)
                 }
             else:
                 self.proxy = {
-                    "http": "http://{}:{}".format(proxy_server, proxy_port),
-                    "https": "https://{}:{}".format(proxy_server, proxy_port),
+                    "http": "http://{}".format(proxy_uri),
+                    "https": "http://{}".format(proxy_uri)
                 }
 
     def get_authorization_url(self, requested_scopes=None,
@@ -500,10 +511,11 @@ class Connection:
             if self.auth_flow_type in ('authorization', 'public'):
                 self.session = self.get_session(state=state,
                                                 redirect_uri=redirect_uri)
-            elif self.auth_flow_type == 'credentials':
+            elif self.auth_flow_type in ('credentials', 'certificate', 'password'):
                 self.session = self.get_session(scopes=scopes)
             else:
-                raise ValueError('"auth_flow_type" must be "authorization", "public" or "credentials"')
+                raise ValueError('"auth_flow_type" must be "authorization", "public", "credentials", "password",'
+                                 ' or "certificate"')
 
         try:
             if self.auth_flow_type == 'authorization':
@@ -511,18 +523,37 @@ class Connection:
                     token_url=self._oauth2_token_url,
                     authorization_response=authorization_url,
                     include_client_id=True,
-                    client_secret=self.auth[1]))
+                    client_secret=self.auth[1],
+                    verify=self.verify_ssl))
             elif self.auth_flow_type == 'public':
                 self.token_backend.token = Token(self.session.fetch_token(
                     token_url=self._oauth2_token_url,
                     authorization_response=authorization_url,
-                    include_client_id=True))
+                    include_client_id=True,
+                    verify=self.verify_ssl))
             elif self.auth_flow_type == 'credentials':
                 self.token_backend.token = Token(self.session.fetch_token(
                     token_url=self._oauth2_token_url,
                     include_client_id=True,
                     client_secret=self.auth[1],
-                    scope=scopes))
+                    scope=scopes,
+                    verify=self.verify_ssl))
+            elif self.auth_flow_type == 'password':
+                self.token_backend.token = Token(self.session.fetch_token(
+                    token_url=self._oauth2_token_url,
+                    include_client_id=True,
+                    username=self.username,
+                    password=self.password,
+                    scope=scopes,
+                    verify=self.verify_ssl))
+            elif self.auth_flow_type == 'certificate':
+                self.token_backend.token = Token(self.session.fetch_token(
+                    token_url=self._oauth2_token_url,
+                    include_client_id=True,
+                    client_assertion=self.auth[1],
+                    client_assertion_type="urn:ietf:params:oauth:client-assertion-type:jwt-bearer",
+                    scope=scopes,
+                    verify=self.verify_ssl))
         except Exception as e:
             log.error('Unable to fetch auth token. Error: {}'.format(str(e)))
             return False
@@ -551,8 +582,10 @@ class Connection:
 
         if self.auth_flow_type in ('authorization', 'public'):
             oauth_client = WebApplicationClient(client_id=client_id)
-        elif self.auth_flow_type == 'credentials':
+        elif self.auth_flow_type in ('credentials', 'certificate'):
             oauth_client = BackendApplicationClient(client_id=client_id)
+        elif self.auth_flow_type == 'password':
+            oauth_client = LegacyApplicationClient(client_id=client_id)
         else:
             raise ValueError('"auth_flow_type" must be "authorization", "credentials" or "public"')
 
@@ -565,7 +598,7 @@ class Connection:
                 raise RuntimeError('No auth token found. Authentication Flow needed')
 
             oauth_client.token = token
-            if self.auth_flow_type in ('authorization', 'public'):
+            if self.auth_flow_type in ('authorization', 'public', 'password'):
                 requested_scopes = None  # the scopes are already in the token (Not if type is backend)
             session = OAuth2Session(client_id=client_id,
                                     client=oauth_client,
@@ -633,22 +666,24 @@ class Connection:
                     self.session.refresh_token(
                         self._oauth2_token_url,
                         client_id=client_id,
-                        client_secret=client_secret)
+                        client_secret=client_secret,
+                        verify=self.verify_ssl)
                 )
-            elif self.auth_flow_type == 'public':
+            elif self.auth_flow_type in ('public', 'password'):
                 client_id = self.auth[0]
                 self.token_backend.token = Token(
                     self.session.refresh_token(
                         self._oauth2_token_url,
-                        client_id=client_id)
+                        client_id=client_id,
+                        verify=self.verify_ssl)
                 )
-            elif self.auth_flow_type == 'credentials':
+            elif self.auth_flow_type in ('credentials', 'certificate'):
                 if self.request_token(None, store_token=False) is False:
                     log.error('Refresh for Client Credentials Grant Flow failed.')
                     return False
             log.debug('New oauth token fetched by refresh method')
         else:
-            log.error('You can not refresh an access token that has no "refreh_token" available.'
+            log.error('You can not refresh an access token that has no "refresh_token" available.'
                       'Include "offline_access" scope when authenticating to get a "refresh_token"')
             return False
 
@@ -660,10 +695,10 @@ class Connection:
         """ Checks if a delay is needed between requests and sleeps if True """
         if self._previous_request_at:
             dif = round(time.time() - self._previous_request_at,
-                        2) * 1000  # difference in miliseconds
+                        2) * 1000  # difference in milliseconds
             if dif < self.requests_delay:
                 sleep_for = (self.requests_delay - dif)
-                log.debug('Sleeping for {} miliseconds'.format(sleep_for))
+                log.debug('Sleeping for {} milliseconds'.format(sleep_for))
                 time.sleep(sleep_for / 1000)  # sleep needs seconds
         self._previous_request_at = time.time()
 
@@ -695,6 +730,8 @@ class Connection:
         if self.timeout is not None:
             kwargs['timeout'] = self.timeout
 
+        kwargs.setdefault("verify", self.verify_ssl)
+
         request_done = False
         token_refreshed = False
 
